; WARNING!!! Don't erase this part before programming. It will erase the
; calibration value stored in 0x01ff.

; PIC10F202 Configuration Bit Settings

; Assembly source line config statements

#include "p10f322.inc"

; CONFIG
; __config 0xFFA0
 __CONFIG _FOSC_INTOSC & _BOREN_OFF & _WDTE_OFF & _PWRTE_ON & _MCLRE_OFF & _CP_OFF & _LVP_OFF & _LPBOR_OFF & _BORV_LO & _WRT_OFF

 ; General Notes:
 ; Internal oscillator is 4MHz. External oscillator disabled.
 ; Watchdog timer is TBD.
 ; BrownOut reset is off for now as is low power BrownOut.
 ; 8 stack registers. 7 nested subroutine calls.
 
 
 ; Pin Assignments:
 ;  1 - GP0: SHUTDN input
 ;  2 - VSS
 ;  3 - GP1: Shutdown output to the base of Q1
 ;  4 - IN2: ADC input.
 ;  5 - VDD
 ;  6 - MCLR_, GP3: unused. tied to VDD in circuit.


;***** Define addresses of the General Purpose Registers ****
; 24 general purpose registers available starting at 0x08
COUNT1	equ 0x40    ;First counter for delay loops
COUNT2	equ 0x41    ;Second counter for delay loops
COUNT3	equ 0x42    ;Third counter for delay loops
SCRATCH	equ 0x43    ;Scratchpad variable for everthing else.

X_SUM equ 0x45    ; rolling average summation - reserve two bytes
X_SUMH equ 0x46
X_AVG	equ 0x47    ; rolling average result LSB
X_AVGH equ 0x48	    ; rolling ave result MSB
FSRX	equ 0x4b
X_BUF	equ 0x4c    ; pointer to the start of the X rolling average registers
Y_SUM equ 0x55    ; rolling average summation - reserve two bytes
Y_SUMH equ 0x56
Y_AVG	equ 0x57    ; rolling average result LSB
Y_AVGH equ 0x58	    ; rolling ave result MSB
FSRY	equ 0x5a
Y_BUF	equ 0x5b    ; pointer to the start of the Y rolling average registers
TEMP	equ 0x64    ; temp register for multiply	


	
	org    0x0000          ; processor reset vector

	movlw	b'01010000' ; Oscillator freq. = 4MHz.
	movwf	OSCCON	    

; **** Disable Interrupts ****
	clrf	INTCON
	clrf	PIE1
	
;**** Disable unused peripherals ****
	movlw	b'11001100' ; disable Wake on Change, disable Weak pullups, 
	movwf	OPTION_REG  ; internal clk -> Timer0, Timer0 source edge don't care,
			    ; Prescaler Assignment -> WDT, Prescaler = 16
			    
;**** Setup PORTA inputs/outputs **** 
	movlw	0x04	    ; Set RA2 as analog input
	movwf	ANSELA	    
	clrf	PORTA	    ; load PORTA register before TRIS assignment
	movlw	0x05	    ; set RA1 as output, RA0 RA2 RA3 are inputs
	movwf	TRISA	    

;*** Setup ADC ***
	movlw	b'00000010' ; FVR disabled temporarily, Temp indicator disabled,
	movwf	FVRCON	    ; ADC FVR = 2.048V   
	movlw	b'01101000' ; Clock:RC Osc, VREF=VDD, input = AN2, ADC disabled
	movwf	ADCON	

Start
	clrwdt		    ; reset watchdog
	call	Delay10ms   
	btfss	PORTA, 0	    ; sample SHUTDN input every 10ms
	goto	Start	    ; back to START if SHUTDN = 0
	movlw	d'5'	    ; number of samples to deglitch SHUTDN input.
	movwf	COUNT3
Sample	call	Delay10ms
	btfss	PORTA, 0
	goto	Start	    ; back to START if SHUTDN = 0
	decfsz	COUNT3, 1   ; SHUTDN must be high 5x in a row
	goto	Sample
	clrwdt		    ; clear the watchdog
	bcf	TRISA, 0    ; make RA0 an output
	bsf	PORTA, 0    ; latch RA0 high to keep LTC4041 in powerfail mode.
	call	Delay20	    ; commit to shutdown at this point. Wait 20 sec.
	call	INIBUF	    ; initialize the rolling average buffers
SCtest	clrwdt		    ; reset watchdog
	call	SCAPCheck   ; test SCAP voltage
	call	div8scaled8 ; divide Y by X (fixed point), divide by 4 and add d180
	btfsc	STATUS,0    ; check for carry bit set	
	goto	SCtest	    ; Yes. Keep testing.
	bsf	PORTA, 1    ; No. Set the shutdown output high
Stuck	clrwdt		    ; reset watchdog	
	goto Stuck	    ; infinite loop until power is cut
	
	

;**** subroutines begin here ****	
Delay1			    ; total count = 1Meg = 1 second
	movlw	d'8'	    ; put 2 into COUNT3
	movwf	COUNT3	    ; COUNT3 x (COUNT1 x 3 + 3)(COUNT2 + 5) = 1001472
	movlw	d'255'	    ; put 255 
	movwf	COUNT1	    ; into COUNT1
Loop2	movlw	d'158'	    ; put 158
	movwf	COUNT2	    ; into COUNT2	    
	
Loop1	decfsz	COUNT1,1    ; counts down from 255
        goto	Loop1       ;
        decfsz	COUNT2,1    ; counts down from 158
	goto	Loop1	    ;
	clrwdt		    ; clear the watchdog timer every 125ms
	decfsz	COUNT3,1    ; counts down from 8
	goto	Loop2	    ; 
	RETLW	0

Delay10ms		    ; delay = 1us x (255x3+3) x (8 + 5) = 9.98ms
	movlw	d'255'
	movwf	COUNT1
	movlw	d'8'
	movwf	COUNT2	    
Loop10
	decfsz	COUNT1,1    
        goto	Loop10       
        decfsz	COUNT2,1    
	goto	Loop10	    
	RETLW	0	
	
Delay20
	movlw	d'20'	    ; 20 x 1 = 20 seconds
	movwf	SCRATCH	    ; into SCRATCH reg.
Loop20	
	call	Delay1	    ; 1 sec delay
	decfsz	SCRATCH,1   ;
        goto	Loop20       ;
	RETLW	0

SCAPCheck
;**** perform ADC conversion on FVR and AN2 ****		
	bsf	FVRCON,7	; FVR enabled
FVRGO	btfss	FVRCON,6	; wait for FVRRDY bit
	goto	FVRGO
	movlw	b'01111101'	; select FVR and turn on ADC
	movwf	ADCON
	NOP			; Acquisition delay
	NOP
	NOP
	NOP
	NOP
	bsf	ADCON,1		; start ADC conversion
W84ADX	btfsc	ADCON,1		; wait for end of conversion
	goto	W84ADX
	bcf	FVRCON,7	; turn off FVR
	call	rollAvgX	; add X result to rolling average
	movlw	b'01101001'	; select AN2 and keep ADC turned on
	movwf	ADCON
	NOP			; Acquisition delay
	NOP
	NOP
	NOP
	NOP
	bsf	ADCON,1		; start ADC conversion
W84ADY	btfsc	ADCON,1		; wait for end of conversion
	goto	W84ADY
	bcf	ADCON,0		; turn off ADC
	call rollAvgY		; add Y result to rolling average
	RETLW	0

; **** Initialize rolling average of battery voltage
INIBUF
	clrf	X_SUM	    ; initialize X_SUM to zero
	clrf	X_SUMH    ; ditto
	movlw	X_BUF	    ; initialize pointer to rolling average block 
	movwf	FSR
CBUFX	clrf	INDF	    ; clear the buffer registers
	incf	FSR,1
	movlw	X_BUF+8   ; X_BUF + 8 = end of buffer
	xorwf	FSR,0
	btfss	STATUS,2    ; end of buffer test
	goto	CBUFX
	movlw	X_BUF	    ; set pointer to X rolling average block 
	movwf	FSRX
	
	clrf	Y_SUM	    ; initialize Y_SUM to zero
	clrf	Y_SUMH    ; ditto
	movlw	Y_BUF	    ; initialize pointer to rolling average block 
	movwf	FSR
CBUFY	clrf	INDF	    ; clear the buffer registers
	incf	FSR,1
	movlw	Y_BUF+8   ; Y_BUF + 8 = end of buffer
	xorwf	FSR,0
	btfss	STATUS,2    ; end of buffer test
	goto	CBUFY
	movlw	Y_BUF	    ; set pointer to Y rolling average block 
	movwf	FSRY
	
	movlw	0x08	    ; put counter into SCRATCH register
	movwf	SCRATCH
Filbuf	call	SCAPCheck    ; check SCAP voltage 8 times to 
	decfsz	SCRATCH,1    ; fill up the rolling average blocks
	goto	Filbuf
	RETLW	0
	
;**** Perform rolling average on ADC data ****
rollAvgX	
	movf	FSRX,0	    ; get current buffer pointer value and
	movwf	FSR	    ; put in FSR
	movf	INDF,0	    ; subtract LSB of old sample from running total
	subwf	X_SUM,1   ;
	btfss	STATUS,0    ; check for borrow from subtraction
	decf	X_SUMH,1  ; subtract borrow from running total MSB  
	movf	ADRES,0     ; put the ADC result in W
	movwf	INDF	    ; put the ADC result into the rolling avg buffer
	addwf	X_SUM,1   ; add ADC result to running total
	btfsc	STATUS,0    ; test for carry from addition
	incf	X_SUMH,1    ; add the carry to the MSB of the running total 
	incf	FSR,1	    ; advance buffer pointer to next sample
	movlw	X_BUF+8   ; smp_buf + 8 = end of buffer
	xorwf	FSR,0	    ; if end of buffer is reached
	movlw	X_BUF
	btfsc	STATUS,2    
	movwf	FSR	    ; reset FSR to start of buffer
	; now divide running total by 8
	movf	X_SUM,0   ;put LSB of running total in W
	movwf	X_AVG	    ; move W in AVERAGE LSB
	movf	X_SUMH,0 ; put MSB of running total in W
	movwf	X_AVGH   ; move W to AVERAGE MSB
	bcf	STATUS,0    ; clear carry bit
	rrf	X_AVGH,1 ; shift right three times:
	rrf	X_AVG,1
	bcf	STATUS,0    ; clear carry bit
	rrf	X_AVGH,1
	rrf	X_AVG,1
	bcf	STATUS,0    ; clear carry bit
	rrf	X_AVGH,1
	rrf	X_AVG,1
	movf	X_AVG,0   ; put average result in W
	movf	FSR,0	    ; store the current pointer address in FSRX
	movwf	FSRX
	RETLW	0

rollAvgY
	movf	FSRY,0	    ; get current buffer pointer value and
	movwf	FSR	    ; put in FSR
	movf	INDF,0	    ; subtract LSB of old sample from running total
	subwf	Y_SUM,1   ;
	btfss	STATUS,0    ; check for borrow from subtraction
	decf	Y_SUMH,1  ; subtract borrow from running total MSB  
	movf	ADRES,0     ; put the ADC result in W
	movwf	INDF	    ; put the ADC result into the rolling avg buffer
	addwf	Y_SUM,1   ; add ADC result to running total
	btfsc	STATUS,0    ; test for carry from addition
	incf	Y_SUMH,1    ; add the carry to the MSB of the running total 
	incf	FSR,1	    ; advance buffer pointer to next sample
	movlw	Y_BUF+8   ; smp_buf + 8 = end of buffer
	xorwf	FSR,0	    ; if end of buffer is reached
	movlw	Y_BUF
	btfsc	STATUS,2    
	movwf	FSR	    ; reset FSR to start of buffer
	; now divide running total by 8
	movf	Y_SUM,0   ;put LSB of running total in W
	movwf	Y_AVG	    ; move W in AVERAGE LSB
	movf	Y_SUMH,0 ; put MSB of running total in W
	movwf	Y_AVGH   ; move W to AVERAGE MSB
	bcf	STATUS,0    ; clear carry bit
	rrf	Y_AVGH,1 ; shift right three times:
	rrf	Y_AVG,1
	bcf	STATUS,0    ; clear carry bit
	rrf	Y_AVGH,1
	rrf	Y_AVG,1
	bcf	STATUS,0    ; clear carry bit
	rrf	Y_AVGH,1
	rrf	Y_AVG,1
	movf	Y_AVG,0   ; put average result in W
	movf	FSR,0	    ; store the current pointer address in FSRY
	movwf	FSRY
	RETLW	0
	
	
; Scaled divider: ((256 * Y_AVG)/X_AVG)/4 + 180
div8scaled8
        clrf TEMP               ;prepare temp regs
        clrf TEMP+1
        clrf TEMP+2
        clrf Y_AVGH
        bsf TEMP, 0             ;set counter bit
        clrc
div8scaled8_loop
        rlf Y_AVG, f		;shift in zero and shift out next divisor bit
        rlf TEMP+1, f
        rlf TEMP+2, f

        movf X_AVG, w		;move X_AVG to w
        btfss TEMP, 0
        goto div8scaled8_add
        subwf TEMP+1, f
        movlw 1
        skpc
        subwf TEMP+2, f
        goto div8scaled8_next

div8scaled8_add
        addwf TEMP+1, f
        movlw 1
        skpnc
        addwf TEMP+2, f

div8scaled8_next
        rlf TEMP, f             ;shift in next result
        rlf Y_AVGH, f
        skpc			;repeat until carry is set
        goto div8scaled8_loop

	movf Y_AVGH, w
	movwf TEMP+1		; TEMP contains result
	
	clrc
	rrf	TEMP+1, f	; divide TEMP by 4
	rrf	TEMP, f
	clrc
	rrf	TEMP+1, f
	rrf	TEMP, f
	movlw	d'180'		; add threshold to W and 
	addwf	TEMP,w		; store in W test Carry bit for high or low
        RETLW	0               ; done!
	
	END


